@@ -69,8 +69,8 @@ void usage(void){
   fprintf(stderr, "usage:\n");
   fprintf(stderr, "  %s write name rnum\n", progname);
   fprintf(stderr, "  %s read name rnum\n", progname);
-  fprintf(stderr, "  %s btwrite name rnum\n", progname);
-  fprintf(stderr, "  %s btread name rnum\n", progname);
+  fprintf(stderr, "  %s btwrite [-rnd] name rnum\n", progname);
+  fprintf(stderr, "  %s btread [-rnd] name rnum\n", progname);
   fprintf(stderr, "\n");
   exit(1);
 }
@@ -292,7 +292,7 @@ int doread(char *name, int rnum){
     key.data = buf;
     key.size = len;
     if(dbp->get(dbp, NULL, &key, &data, 0) != 0){
-      fprintf(stderr, "DB->put failed\n");
+      fprintf(stderr, "DB->get failed\n");
       err = TRUE;
       break;
     }
@@ -409,7 +409,7 @@ int dobtread(char *name, int rnum, int rnd){
     key.data = buf;
     key.size = len;
     if(dbp->get(dbp, NULL, &key, &data, 0) != 0){
-      fprintf(stderr, "DB->put failed\n");
+      fprintf(stderr, "DB->get failed\n");
       err = TRUE;
       break;
     }
