@@ -363,7 +363,7 @@ int dostrstr(int rnum, int disp){
     }
     if(disp && std) printf("%s\n", buf);
   }
-  if(err) fprintf(stderr, "%s: sorting failed\n", progname);
+  if(err) fprintf(stderr, "%s: string scanning failed\n", progname);
   if(!disp && !err){
     printfflush("ok\n");
     printfflush("all ok\n\n");
@@ -550,6 +550,13 @@ int domisc(void){
   const char *op, *np;
   time_t t;
   printfflush("<Miscellaneous Test>\n\n");
+  printfflush("Checking memory allocation ... ");
+  tmp = cbmalloc(1024);
+  for(i = 1; i <= 65536; i *= 2){
+    tmp = cbrealloc(tmp, i);
+  }
+  cbfree(tmp);
+  printfflush("ok\n");
   printfflush("Checking basic datum ... ");
   odatum = cbdatumopen("x", -1);
   for(i = 0; i < 1000; i++){
@@ -699,6 +706,12 @@ int domisc(void){
     fprintf(stderr, "%s: cbstrsqzspc is invalid\n", progname);
     return 1;
   }
+  cbstrcututf(tmp, 5);
+  if(cbstrcountutf(tmp) != 5){
+    free(tmp);
+    fprintf(stderr, "%s: cbstrcututf or cbstrcountutf is invalid\n", progname);
+    return 1;
+  }
   free(tmp);
   printfflush("ok\n");
   printfflush("Checking encoding utilities ... ");
@@ -801,7 +814,10 @@ int domisc(void){
       sprintf(kbuf, "%08d", j);
       cbmapput(gmap, kbuf, -1, kbuf, -1, TRUE);
     }
-    if(myrand() % 64 == 0) cbggcsweep();
+    if(myrand() % 64 == 0){
+      cbvmemavail(100);
+      cbggcsweep();
+    }
   }
   printfflush("ok\n");
   printfflush("all ok\n\n");
