@@ -100,7 +100,8 @@ CURIA *cropen(const char *name, int omode, int bnum, int dnum){
         return NULL;
       }
     } else {
-      if(!dpput(attr, CR_KEYDNUM, -1, (char *)&dnum, sizeof(int), DP_DOVER)){
+      if(!dpput(attr, CR_KEYDNUM, -1, (char *)&dnum, sizeof(int), DP_DOVER) ||
+         !dpput(attr, CR_KEYLRNUM, -1, (char *)&lrnum, sizeof(int), DP_DOVER)){
         dpclose(attr);
         return NULL;
       }
@@ -218,6 +219,7 @@ int crput(CURIA *curia, const char *kbuf, int ksiz, const char *vbuf, int vsiz,
     dpecodeset(DP_EMODE, __FILE__, __LINE__);
     return FALSE;
   }
+  if(ksiz < 0) ksiz = strlen(kbuf);
   switch(dmode){
   case CR_DKEEP: dpdmode = DP_DKEEP; break;
   case CR_DCAT: dpdmode = DP_DCAT; break;
@@ -236,6 +238,7 @@ int crout(CURIA *curia, const char *kbuf, int ksiz){
     dpecodeset(DP_EMODE, __FILE__, __LINE__);
     return FALSE;
   }
+  if(ksiz < 0) ksiz = strlen(kbuf);
   tnum = dpouterhash(kbuf, ksiz) % curia->dnum;
   return dpout(curia->depots[tnum], kbuf, ksiz);
 }
@@ -245,6 +248,7 @@ int crout(CURIA *curia, const char *kbuf, int ksiz){
 char *crget(CURIA *curia, const char *kbuf, int ksiz, int start, int max, int *sp){
   int tnum;
   assert(curia && kbuf && start >= 0);
+  if(ksiz < 0) ksiz = strlen(kbuf);
   tnum = dpouterhash(kbuf, ksiz) % curia->dnum;
   return dpget(curia->depots[tnum], kbuf, ksiz, start, max, sp);
 }
@@ -254,6 +258,7 @@ char *crget(CURIA *curia, const char *kbuf, int ksiz, int start, int max, int *s
 int crgetwb(CURIA *curia, const char *kbuf, int ksiz, int start, int max, char *vbuf){
   int tnum;
   assert(curia && kbuf && start >= 0 && max >= 0 && vbuf);
+  if(ksiz < 0) ksiz = strlen(kbuf);
   tnum = dpouterhash(kbuf, ksiz) % curia->dnum;
   return dpgetwb(curia->depots[tnum], kbuf, ksiz, start, max, vbuf);
 }
@@ -263,6 +268,7 @@ int crgetwb(CURIA *curia, const char *kbuf, int ksiz, int start, int max, char *
 int crvsiz(CURIA *curia, const char *kbuf, int ksiz){
   int tnum;
   assert(curia && kbuf);
+  if(ksiz < 0) ksiz = strlen(kbuf);
   tnum = dpouterhash(kbuf, ksiz) % curia->dnum;
   return dpvsiz(curia->depots[tnum], kbuf, ksiz);
 }
@@ -686,6 +692,26 @@ int crimportdb(CURIA *curia, const char *name){
 }
 
 
+/* Retrieve a record directly from a database directory. */
+char *crsnaffle(const char *name, const char *kbuf, int ksiz, int *sp){
+  char path[CR_PATHBUFSIZ], *vbuf;
+  int dnum, vsiz, tnum;
+  assert(name && kbuf);
+  if(ksiz < 0) ksiz = strlen(kbuf);
+  sprintf(path, "%s%c%s", name, MYPATHCHR, CR_DPNAME);
+  if(!(vbuf = dpsnaffle(path, CR_KEYDNUM, -1, &vsiz)) || vsiz != sizeof(int) ||
+     (dnum = *(int *)vbuf) < 1){
+    free(vbuf);
+    dpecodeset(DP_EBROKEN, __FILE__, __LINE__);
+    return NULL;
+  }
+  free(vbuf);
+  tnum = dpouterhash(kbuf, ksiz) % dnum;
+  sprintf(path, "%s%c%04d%c%s", name, MYPATHCHR, tnum + 1, MYPATHCHR, CR_DPNAME);
+  return dpsnaffle(path, kbuf, ksiz, sp);
+}
+
+
 /* Store a large object. */
 int crputlob(CURIA *curia, const char *kbuf, int ksiz, const char *vbuf, int vsiz, int dmode){
   char *path;
@@ -872,6 +898,27 @@ int crmemsync(CURIA *curia){
 }
 
 
+/* Synchronize updating contents on memory, not physically. */
+int crmemflush(CURIA *curia){
+  int i, err;
+  assert(curia);
+  if(!curia->wmode){
+    dpecodeset(DP_EMODE, __FILE__, __LINE__);
+    return FALSE;
+  }
+  err = FALSE;
+  if(!dpput(curia->attr, CR_KEYLRNUM, -1, (char *)&(curia->lrnum), sizeof(int), DP_DOVER) ||
+     !dpmemsync(curia->attr)) err = TRUE;
+  for(i = 0; i < curia->dnum; i++){
+    if(!dpmemflush(curia->depots[i])){
+      err = TRUE;
+      break;
+    }
+  }
+  return err ? FALSE : TRUE;
+}
+
+
 /* Get flags of a database. */
 int crgetflags(CURIA *curia){
   assert(curia);
