@@ -86,6 +86,8 @@ static int dprecappend(DEPOT *depot, const char *kbuf, int ksiz, const char *vbu
                        int hash, int left, int right);
 static int dprecover(DEPOT *depot, int off, int *head, const char *vbuf, int vsiz, int cat);
 static int dprecdelete(DEPOT *depot, int off, int *head, int reusable);
+static void dpfbpoolcoal(DEPOT *depot);
+static int dpfbpoolcmp(const void *a, const void *b);
 
 
 
@@ -211,9 +213,10 @@ DEPOT *dpopen(const char *name, int omode, int bnum){
     dpecodeset(DP_EBROKEN, __FILE__, __LINE__);
     return NULL;
   }
-  if((dpbigendian() ? memcmp(hbuf, DP_MAGICNUMB, strlen(DP_MAGICNUMB)) != 0 :
-      memcmp(hbuf, DP_MAGICNUML, strlen(DP_MAGICNUML)) != 0) ||
-     *((int *)(hbuf + DP_FSIZOFF)) != fsiz){
+  if(!(omode & DP_ONOLCK) &&
+     ((dpbigendian() ? memcmp(hbuf, DP_MAGICNUMB, strlen(DP_MAGICNUMB)) != 0 :
+       memcmp(hbuf, DP_MAGICNUML, strlen(DP_MAGICNUML)) != 0) ||
+      *((int *)(hbuf + DP_FSIZOFF)) != fsiz)){
     close(fd);
     dpecodeset(DP_EBROKEN, __FILE__, __LINE__);
     return NULL;
@@ -263,6 +266,7 @@ DEPOT *dpopen(const char *name, int omode, int bnum){
     depot->fbpool[i+1] = -1;
   }
   depot->fbpsiz = DP_FBPOOLSIZ * 2;
+  depot->fbpinc = 0;
   depot->align = 0;
   return depot;
 }
@@ -1226,6 +1230,119 @@ int dpimportdb(DEPOT *depot, const char *name){
 }
 
 
+/* Retrieve a record directly from a database file. */
+char *dpsnaffle(const char *name, const char* kbuf, int ksiz, int *sp){
+  char hbuf[DP_HEADSIZ], *map, *vbuf, *tkbuf;
+  int fd, fsiz, bnum, msiz, *buckets, hash, thash, head[DP_RHNUM], err, off, vsiz, tksiz, kcmp;
+  struct stat sbuf;
+  assert(name && kbuf);
+  if(ksiz < 0) ksiz = strlen(kbuf);
+  if((fd = open(name, O_RDONLY, DP_FILEMODE)) == -1){
+    dpecodeset(DP_EOPEN, __FILE__, __LINE__);
+    return NULL;
+  }
+  if(fstat(fd, &sbuf) == -1 || !S_ISREG(sbuf.st_mode)){
+    close(fd);
+    dpecodeset(DP_ESTAT, __FILE__, __LINE__);
+    return NULL;
+  }
+  fsiz = sbuf.st_size;
+  if(!dpseekread(fd, 0, hbuf, DP_HEADSIZ)){
+    close(fd);
+    dpecodeset(DP_EBROKEN, __FILE__, __LINE__);
+    return NULL;
+  }
+  if(dpbigendian() ? memcmp(hbuf, DP_MAGICNUMB, strlen(DP_MAGICNUMB)) != 0 :
+     memcmp(hbuf, DP_MAGICNUML, strlen(DP_MAGICNUML)) != 0){
+    close(fd);
+    dpecodeset(DP_EBROKEN, __FILE__, __LINE__);
+    return NULL;
+  }
+  bnum = *((int *)(hbuf + DP_BNUMOFF));
+  if(bnum < 1 || fsiz < DP_HEADSIZ + bnum * sizeof(int)){
+    close(fd);
+    dpecodeset(DP_EBROKEN, __FILE__, __LINE__);
+    return NULL;
+  }
+  msiz = DP_HEADSIZ + bnum * sizeof(int);
+  map = mmap(0, msiz, PROT_READ, MAP_SHARED, fd, 0);
+  if(map == MAP_FAILED){
+    close(fd);
+    dpecodeset(DP_EMAP, __FILE__, __LINE__);
+    return NULL;
+  }
+  buckets = (int *)(map + DP_HEADSIZ);
+  err = FALSE;
+  vbuf = NULL;
+  vsiz = 0;
+  hash = dpsecondhash(kbuf, ksiz);
+  off = buckets[dpfirsthash(kbuf, ksiz)%bnum];
+  while(off != 0){
+    if(!dpseekread(fd, off, head, DP_RHNUM * sizeof(int))){
+      err = TRUE;
+      break;
+    }
+    if(head[DP_RHIKSIZ] < 0 || head[DP_RHIVSIZ] < 0 || head[DP_RHIPSIZ] < 0 ||
+       head[DP_RHILEFT] < 0 || head[DP_RHIRIGHT] < 0){
+      dpecodeset(DP_EBROKEN, __FILE__, __LINE__);
+      err = TRUE;
+      break;
+    }
+    thash = head[DP_RHIHASH];
+    if(hash > thash){
+      off = head[DP_RHILEFT];
+    } else if(hash < thash){
+      off = head[DP_RHIRIGHT];
+    } else {
+      tksiz = head[DP_RHIKSIZ];
+      if(!(tkbuf = malloc(tksiz + 1))){
+        dpecodeset(DP_EALLOC, __FILE__, __LINE__);
+        err = TRUE;
+        break;
+      }
+      if(!dpseekread(fd, off + DP_RHNUM * sizeof(int), tkbuf, tksiz)){
+        free(tkbuf);
+        err = TRUE;
+        break;
+      }
+      tkbuf[tksiz] = '\0';
+      kcmp = dpkeycmp(kbuf, ksiz, tkbuf, tksiz);
+      free(tkbuf);
+      if(kcmp > 0){
+        off = head[DP_RHILEFT];
+      } else if(kcmp < 0){
+        off = head[DP_RHIRIGHT];
+      } else if(head[DP_RHIFLAGS] & DP_RECFDEL){
+        break;
+      } else {
+        vsiz = head[DP_RHIVSIZ];
+        if(!(vbuf = malloc(vsiz + 1))){
+          dpecodeset(DP_EALLOC, __FILE__, __LINE__);
+          err = TRUE;
+          break;
+        }
+        if(!dpseekread(fd, off + DP_RHNUM * sizeof(int) + head[DP_RHIKSIZ], vbuf, vsiz)){
+          free(vbuf);
+          vbuf = NULL;
+          err = TRUE;
+          break;
+        }
+        vbuf[vsiz] = '\0';
+        break;
+      }
+    }
+  }
+  if(vbuf){
+    if(sp) *sp = vsiz;
+  } else if(!err){
+    dpecodeset(DP_ENOITEM, __FILE__, __LINE__);
+  }
+  munmap(map, msiz);
+  close(fd);
+  return vbuf;
+}
+
+
 /* Hash function used inside Depot. */
 int dpinnerhash(const char *kbuf, int ksiz){
   assert(kbuf);
@@ -1318,6 +1435,28 @@ int dpmemsync(DEPOT *depot){
 }
 
 
+/* Synchronize updating contents on memory, not physically. */
+int dpmemflush(DEPOT *depot){
+  assert(depot);
+  if(depot->fatal){
+    dpecodeset(DP_EFATAL, __FILE__, __LINE__);
+    return FALSE;
+  }
+  if(!depot->wmode){
+    dpecodeset(DP_EMODE, __FILE__, __LINE__);
+    return FALSE;
+  }
+  *((int *)(depot->map + DP_FSIZOFF)) = depot->fsiz;
+  *((int *)(depot->map + DP_RNUMOFF)) = depot->rnum;
+  if(mflush(depot->map, depot->msiz, MS_SYNC) == -1){
+    dpecodeset(DP_EMAP, __FILE__, __LINE__);
+    depot->fatal = TRUE;
+    return FALSE;
+  }
+  return TRUE;
+}
+
+
 /* Get flags of a database. */
 int dpgetflags(DEPOT *depot){
   assert(depot);
@@ -1674,7 +1813,7 @@ static int dprechead(DEPOT *depot, int off, int *head, char *ebuf, int *eep){
       *eep = TRUE;
       if(!dpseekread(depot->fd, off, ebuf, DP_ENTBUFSIZ)) return FALSE;
       memcpy(head, ebuf, DP_RHNUM * sizeof(int));
-      if(head[DP_RHIKSIZ] < 0 || head[DP_RHIVSIZ] < 0 || DP_RHIPSIZ < 0 ||
+      if(head[DP_RHIKSIZ] < 0 || head[DP_RHIVSIZ] < 0 || head[DP_RHIPSIZ] < 0 ||
          head[DP_RHILEFT] < 0 || head[DP_RHIRIGHT] < 0){
         dpecodeset(DP_EBROKEN, __FILE__, __LINE__);
         return FALSE;
@@ -1683,7 +1822,7 @@ static int dprechead(DEPOT *depot, int off, int *head, char *ebuf, int *eep){
     }
   }
   if(!dpseekread(depot->fd, off, head, DP_RHNUM * sizeof(int))) return FALSE;
-  if(head[DP_RHIKSIZ] < 0 || head[DP_RHIVSIZ] < 0 || DP_RHIPSIZ < 0 ||
+  if(head[DP_RHIKSIZ] < 0 || head[DP_RHIVSIZ] < 0 || head[DP_RHIPSIZ] < 0 ||
      head[DP_RHILEFT] < 0 || head[DP_RHIRIGHT] < 0){
     dpecodeset(DP_EBROKEN, __FILE__, __LINE__);
     return FALSE;
@@ -1862,32 +2001,66 @@ static int dprecsearch(DEPOT *depot, const char *kbuf, int ksiz, int hash, int *
 static int dprecrewrite(DEPOT *depot, int off, int rsiz, const char *kbuf, int ksiz,
                         const char *vbuf, int vsiz, int hash, int left, int right){
   char ebuf[DP_WRTBUFSIZ];
-  int head[DP_RHNUM], asiz, psiz, hoff, koff, voff, c;
+  int i, head[DP_RHNUM], asiz, hoff, koff, voff, mi, min, size;
   assert(depot && off >= 1 && rsiz > 0 && kbuf && ksiz >= 0 && vbuf && vsiz >= 0);
-  psiz = rsiz - sizeof(head) - ksiz - vsiz;
   head[DP_RHIFLAGS] = 0;
   head[DP_RHIHASH] = hash;
   head[DP_RHIKSIZ] = ksiz;
   head[DP_RHIVSIZ] = vsiz;
-  head[DP_RHIPSIZ] = psiz;
+  head[DP_RHIPSIZ] = rsiz - sizeof(head) - ksiz - vsiz;;
   head[DP_RHILEFT] = left;
   head[DP_RHIRIGHT] = right;
-  asiz = sizeof(head) + ksiz + vsiz + psiz;
+  asiz = sizeof(head) + ksiz + vsiz;
+  if(depot->fbpsiz > DP_FBPOOLSIZ * 4 && head[DP_RHIPSIZ] > asiz){
+    rsiz = (head[DP_RHIPSIZ] - asiz) / 2 + asiz;
+    head[DP_RHIPSIZ] -= rsiz;
+  } else {
+    rsiz = 0;
+  }
   if(asiz <= DP_WRTBUFSIZ){
     memcpy(ebuf, head, sizeof(head));
     memcpy(ebuf + sizeof(head), kbuf, ksiz);
     memcpy(ebuf + sizeof(head) + ksiz, vbuf, vsiz);
-    memset(ebuf + sizeof(head) + ksiz + vsiz, 0, psiz);
     if(!dpseekwrite(depot->fd, off, ebuf, asiz)) return FALSE;
   } else {
     hoff = off;
     koff = hoff + sizeof(head);
     voff = koff + ksiz;
-    c = 0;
     if(!dpseekwrite(depot->fd, hoff, head, sizeof(head)) ||
-       !dpseekwrite(depot->fd, koff, kbuf, ksiz) ||
-       !dpseekwrite(depot->fd, voff, vbuf, vsiz) ||
-       (psiz > 0 && !dpseekwrite(depot->fd, voff + vsiz + psiz - 1, &c, 1))) return FALSE;
+       !dpseekwrite(depot->fd, koff, kbuf, ksiz) || !dpseekwrite(depot->fd, voff, vbuf, vsiz))
+      return FALSE;
+  }
+  if(rsiz > 0){
+    off += sizeof(head) + ksiz + vsiz + head[DP_RHIPSIZ];
+    head[DP_RHIFLAGS] = DP_RECFDEL | DP_RECFREUSE;
+    head[DP_RHIHASH] = hash;
+    head[DP_RHIKSIZ] = ksiz;
+    head[DP_RHIVSIZ] = vsiz;
+    head[DP_RHIPSIZ] = rsiz - sizeof(head) - ksiz - vsiz;;
+    head[DP_RHILEFT] = 0;
+    head[DP_RHIRIGHT] = 0;
+    if(!dpseekwrite(depot->fd, off, head, sizeof(head))) return FALSE;
+    size = dprecsize(head);
+    mi = -1;
+    min = -1;
+    for(i = 0; i < depot->fbpsiz; i += 2){
+      if(depot->fbpool[i] == -1){
+        depot->fbpool[i] = off;
+        depot->fbpool[i+1] = size;
+        dpfbpoolcoal(depot);
+        mi = -1;
+        break;
+      }
+      if(mi == -1 || depot->fbpool[i+1] < min){
+        mi = i;
+        min = depot->fbpool[i+1];
+      }
+    }
+    if(mi >= 0 && size > min){
+      depot->fbpool[mi] = off;
+      depot->fbpool[mi+1] = size;
+      dpfbpoolcoal(depot);
+    }
   }
   return TRUE;
 }
@@ -1993,12 +2166,14 @@ static int dprecdelete(DEPOT *depot, int off, int *head, int reusable){
   int i, mi, min, size;
   assert(depot && off >= 0 && head);
   if(reusable){
+    size = dprecsize(head);
     mi = -1;
     min = -1;
     for(i = 0; i < depot->fbpsiz; i += 2){
       if(depot->fbpool[i] == -1){
         depot->fbpool[i] = off;
-        depot->fbpool[i+1] = dprecsize(head);
+        depot->fbpool[i+1] = size;
+        dpfbpoolcoal(depot);
         mi = -1;
         break;
       }
@@ -2007,9 +2182,10 @@ static int dprecdelete(DEPOT *depot, int off, int *head, int reusable){
         min = depot->fbpool[i+1];
       }
     }
-    if(mi >= 0 && (size = dprecsize(head)) > min){
+    if(mi >= 0 && size > min){
       depot->fbpool[mi] = off;
       depot->fbpool[mi+1] = size;
+      dpfbpoolcoal(depot);
     }
   }
   return dpseekwritenum(depot->fd, off + DP_RHIFLAGS * sizeof(int),
@@ -2017,5 +2193,35 @@ static int dprecdelete(DEPOT *depot, int off, int *head, int reusable){
 }
 
 
+/* Make contiguous records of the free block pool coalesce.
+   `depot' specifies a database handle. */
+static void dpfbpoolcoal(DEPOT *depot){
+  int i;
+  assert(depot);
+  if(depot->fbpinc++ <= depot->fbpsiz / 4) return;
+  depot->fbpinc = 0;
+  qsort(depot->fbpool, depot->fbpsiz / 2, sizeof(int) * 2, dpfbpoolcmp);
+  for(i = 2; i < depot->fbpsiz; i += 2){
+    if(depot->fbpool[i-2] > 0 &&
+       depot->fbpool[i-2] + depot->fbpool[i-1] - depot->fbpool[i] == 0){
+      depot->fbpool[i] = depot->fbpool[i-2];
+      depot->fbpool[i+1] += depot->fbpool[i-1];
+      depot->fbpool[i-2] = -1;
+      depot->fbpool[i-1] = -1;
+    }
+  }
+}
+
+
+/* Compare two records of the free block pool.
+   `a' specifies the pointer to one record.
+   `b' specifies the pointer to the other record.
+   The return value is 0 if two equals, positive if the formar is big, else, negative. */
+static int dpfbpoolcmp(const void *a, const void *b){
+  assert(a && b);
+  return *(int *)a - *(int *)b;
+}
+
+
 
 /* END OF FILE */
