@@ -431,7 +431,7 @@ public class Villa implements ADBM {
     }
   }
   /**
-   * Move the cursor to position around a record.
+   * Move the cursor to a position around a record.
    * @param key a byte array of a key.
    * @param jmode detail adjustment: `Villa.JFORWARD', which means that the cursor is set to
    * the first record of the same key and that the cursor is set to the next substitute if
@@ -447,7 +447,7 @@ public class Villa implements ADBM {
     }
   }
   /**
-   * Move the cursor to position around a record for stepping forward.
+   * Move the cursor to a position around a record for stepping forward.
    * The same as `curjump(key, Villa.JFORFARD)'.
    * @see #curjump(byte[], int)
    */
@@ -455,7 +455,7 @@ public class Villa implements ADBM {
     curjump(key, JFORWARD);
   }
   /**
-   * Move the cursor to position around a record composed of serializable objects.
+   * Move the cursor to a position around a record composed of serializable objects.
    * The same as `curjump(qdbm.Util.serialize(key), jmode)'.
    * @see #curjump(byte[], int)
    * @note If serialization is failed, an instance of `VillaException' is thrown.
@@ -836,8 +836,28 @@ public class Villa implements ADBM {
   public boolean error() throws VillaException {
     return fatalerror();
   }
+  /**
+   * Get a multiple cursor.
+   * @return a multiple cursor.
+   * @note Even if plural cursors are fetched out of a database handle, they does not share the
+   * locations with each other.  Note that this method can be used only if the database handle
+   * is connected as a reader.
+   */
+  public VillaCursor mulcuropen() throws VillaException {
+    return new VillaCursor(this);
+  }
   //----------------------------------------------------------------
-  // private methods
+  // friendly methods
+  //----------------------------------------------------------------
+  /**
+   * Get the index of the native table for database handles.
+   * @return the index of the native table for database handles.
+   */
+  public int getindex(){
+    return index;
+  }
+  //----------------------------------------------------------------
+  // private static methods
   //----------------------------------------------------------------
   /**
    * Compare keys of two records as serialized objects implementing `java.util.Comparable'.
