@@ -873,7 +873,7 @@ int odmerge(const char *name, const CBLIST *elemnames){
       if(vsiz > sizeof(ODPAIR)){
         pairs = (ODPAIR *)vbuf;
         pnum = vsiz / sizeof(ODPAIR);
-        cbqsort(pairs, pnum, sizeof(ODPAIR), odsortcompare);
+        qsort(pairs, pnum, sizeof(ODPAIR), odsortcompare);
         if(!crput(curia, word, wsiz, vbuf, vsiz, CR_DOVER)) err = TRUE;
       }
       free(vbuf);
@@ -1104,7 +1104,7 @@ CBMAP *oddocscores(const ODDOC *doc, int max, ODEUM *odeum){
     owords[i].word = word;
     owords[i].num = *(int *)cbmapget(map, word, wsiz, NULL);
   }
-  cbqsort(owords, mnum, sizeof(ODWORD), odwordcompare);
+  qsort(owords, mnum, sizeof(ODWORD), odwordcompare);
   if(odeum){
     if(mnum > max * OD_KEYCRATIO) mnum = (int)(max * OD_KEYCRATIO);
     for(i = 0; i < mnum; i++){
@@ -1114,7 +1114,7 @@ CBMAP *oddocscores(const ODDOC *doc, int max, ODEUM *odeum){
       if(ival < 8.0) ival = 8.0;
       owords[i].num = (int)(owords[i].num / ival);
     }
-    cbqsort(owords, mnum, sizeof(ODWORD), odwordcompare);
+    qsort(owords, mnum, sizeof(ODWORD), odwordcompare);
   }
   if(mnum > max) mnum = max;
   kwmap = cbmapopenex(OD_MAPPBNUM);
@@ -1210,7 +1210,7 @@ ODPAIR *odpairsand(ODPAIR *apairs, int anum, ODPAIR *bpairs, int bnum, int *np){
     rnum++;
   }
   cbmapclose(map);
-  cbqsort(result, rnum, sizeof(ODPAIR), odsortcompare);
+  qsort(result, rnum, sizeof(ODPAIR), odsortcompare);
   *np = rnum;
   return result;
 }
@@ -1240,7 +1240,7 @@ ODPAIR *odpairsor(ODPAIR *apairs, int anum, ODPAIR *bpairs, int bnum, int *np){
     result[i].score = *(int *)cbmapget(map, tmp, sizeof(int), NULL);
   }
   cbmapclose(map);
-  cbqsort(result, rnum, sizeof(ODPAIR), odsortcompare);
+  qsort(result, rnum, sizeof(ODPAIR), odsortcompare);
   *np = rnum;
   return result;
 }
@@ -1263,7 +1263,7 @@ ODPAIR *odpairsnotand(ODPAIR *apairs, int anum, ODPAIR *bpairs, int bnum, int *n
     rnum++;
   }
   cbmapclose(map);
-  cbqsort(result, rnum, sizeof(ODPAIR), odsortcompare);
+  qsort(result, rnum, sizeof(ODPAIR), odsortcompare);
   *np = rnum;
   return result;
 }
@@ -1272,7 +1272,7 @@ ODPAIR *odpairsnotand(ODPAIR *apairs, int anum, ODPAIR *bpairs, int bnum, int *n
 /* Sort a set of documents in descending order of scores. */
 void odpairssort(ODPAIR *pairs, int pnum){
   assert(pairs && pnum >= 0);
-  cbqsort(pairs, pnum, sizeof(ODPAIR), odsortcompare);
+  qsort(pairs, pnum, sizeof(ODPAIR), odsortcompare);
 }
 
 
@@ -1776,7 +1776,7 @@ static int odsortindex(ODEUM *odeum, const char *fname){
     if((tmp = crget(odeum->indexdb, word, wsiz, 0, -1, &tsiz)) != NULL){
       if(tsiz > sizeof(ODPAIR)){
         pairs = (ODPAIR *)tmp;
-        cbqsort(pairs, tsiz / sizeof(ODPAIR), sizeof(ODPAIR), odsortcompare);
+        qsort(pairs, tsiz / sizeof(ODPAIR), sizeof(ODPAIR), odsortcompare);
         if(!crput(odeum->indexdb, word, wsiz, tmp, tsiz, CR_DOVER)){
           free(tmp);
           return FALSE;
