@@ -32,7 +32,7 @@
 #define VL_CACHEOUT    8                 /* number of pages in a process of cacheout */
 #define VL_INITBNUM    32749             /* initial bucket number */
 #define VL_PAGEALIGN   -3                /* alignment for pages */
-#define VL_FBPOOLSIZ   128               /* size of free block pool */
+#define VL_FBPOOLSIZ   256               /* size of free block pool */
 #define VL_PATHBUFSIZ  1024              /* size of a path buffer */
 #define VL_TMPFSUF     MYEXTSTR "vltmp"  /* suffix of a temporary file */
 #define VL_ROOTKEY     -1                /* key of the root key */
@@ -658,7 +658,7 @@ int vlcurnext(VILLA *villa){
 }
 
 
-/* Move the cursor to position around a record. */
+/* Move the cursor to a position around a record. */
 int vlcurjump(VILLA *villa, const char *kbuf, int ksiz, int jmode){
   VLLEAF *leaf;
   VLREC *recp;
@@ -906,38 +906,23 @@ void vlsettuning(VILLA *villa, int lrecmax, int nidxmax, int lcnum, int ncnum){
 }
 
 
-/* Synchronize updating contents with the file and the device. */
-int vlsync(VILLA *villa){
-  int err, pid;
-  const char *tmp;
-  assert(villa);
+/* Set the size of the free block pool of a database handle. */
+int vlsetfbpsiz(VILLA *villa, int size){
+  assert(villa && size >= 0);
   if(!villa->wmode){
     dpecodeset(DP_EMODE, __FILE__, __LINE__);
     return FALSE;
   }
-  if(villa->tran){
-    dpecodeset(DP_EMISC, __FILE__, __LINE__);
-    return FALSE;
-  }
+  return dpsetfbpsiz(villa->depot, size);
+}
+
+
+/* Synchronize updating contents with the file and the device. */
+int vlsync(VILLA *villa){
+  int err;
   err = FALSE;
-  cbmapiterinit(villa->leafc);
-  while((tmp = cbmapiternext(villa->leafc, NULL)) != NULL){
-    pid = *(int *)tmp;
-    if(!vlleafcacheout(villa, pid)) err = TRUE;
-  }
-  cbmapiterinit(villa->nodec);
-  while((tmp = cbmapiternext(villa->nodec, NULL)) != NULL){
-    pid = *(int *)tmp;
-    if(!vlnodecacheout(villa, pid)) err = TRUE;
-  }
-  if(!dpsetalign(villa->depot, 0)) err = TRUE;
-  if(!vldpputnum(villa->depot, VL_ROOTKEY, villa->root)) err = TRUE;
-  if(!vldpputnum(villa->depot, VL_LASTKEY, villa->last)) err = TRUE;
-  if(!vldpputnum(villa->depot, VL_LNUMKEY, villa->lnum)) err = TRUE;
-  if(!vldpputnum(villa->depot, VL_NNUMKEY, villa->nnum)) err = TRUE;
-  if(!vldpputnum(villa->depot, VL_RNUMKEY, villa->rnum)) err = TRUE;
+  if(!vlmemsync(villa)) err = TRUE;
   if(!dpsync(villa->depot)) err = TRUE;
-  if(!dpsetalign(villa->depot, VL_PAGEALIGN)) err = TRUE;
   return err ? FALSE : TRUE;
 }
 
@@ -1043,17 +1028,13 @@ int vltranbegin(VILLA *villa){
   while((tmp = cbmapiternext(villa->leafc, NULL)) != NULL){
     pid = *(int *)tmp;
     leaf = (VLLEAF *)cbmapget(villa->leafc, (char *)&pid, sizeof(int), NULL);
-    if(leaf->dirty){
-      if(!vlleafsave(villa, leaf)) err = TRUE;
-    }
+    if(leaf->dirty && !vlleafsave(villa, leaf)) err = TRUE;
   }
   cbmapiterinit(villa->nodec);
   while((tmp = cbmapiternext(villa->nodec, NULL)) != NULL){
     pid = *(int *)tmp;
     node = (VLNODE *)cbmapget(villa->nodec, (char *)&pid, sizeof(int), NULL);
-    if(node->dirty){
-      if(!vlnodesave(villa, node)) err = TRUE;
-    }
+    if(node->dirty && !vlnodesave(villa, node)) err = TRUE;
   }
   if(!dpsetalign(villa->depot, 0)) err = TRUE;
   if(!vldpputnum(villa->depot, VL_ROOTKEY, villa->root)) err = TRUE;
@@ -1093,17 +1074,13 @@ int vltrancommit(VILLA *villa){
   while((tmp = cbmapiternext(villa->leafc, NULL)) != NULL){
     pid = *(int *)tmp;
     leaf = (VLLEAF *)cbmapget(villa->leafc, (char *)&pid, sizeof(int), NULL);
-    if(leaf->dirty){
-      if(!vlleafsave(villa, leaf)) err = TRUE;
-    }
+    if(leaf->dirty && !vlleafsave(villa, leaf)) err = TRUE;
   }
   cbmapiterinit(villa->nodec);
   while((tmp = cbmapiternext(villa->nodec, NULL)) != NULL){
     pid = *(int *)tmp;
     node = (VLNODE *)cbmapget(villa->nodec, (char *)&pid, sizeof(int), NULL);
-    if(node->dirty){
-      if(!vlnodesave(villa, node)) err = TRUE;
-    }
+    if(node->dirty && !vlnodesave(villa, node)) err = TRUE;
   }
   if(!dpsetalign(villa->depot, 0)) err = TRUE;
   if(!vldpputnum(villa->depot, VL_ROOTKEY, villa->root)) err = TRUE;
@@ -1356,6 +1333,78 @@ int vlimportdb(VILLA *villa, const char *name){
  *************************************************************************************************/
 
 
+/* Synchronize updating contents on memory. */
+int vlmemsync(VILLA *villa){
+  int err, pid;
+  const char *tmp;
+  assert(villa);
+  if(!villa->wmode){
+    dpecodeset(DP_EMODE, __FILE__, __LINE__);
+    return FALSE;
+  }
+  if(villa->tran){
+    dpecodeset(DP_EMISC, __FILE__, __LINE__);
+    return FALSE;
+  }
+  err = FALSE;
+  cbmapiterinit(villa->leafc);
+  while((tmp = cbmapiternext(villa->leafc, NULL)) != NULL){
+    pid = *(int *)tmp;
+    if(!vlleafcacheout(villa, pid)) err = TRUE;
+  }
+  cbmapiterinit(villa->nodec);
+  while((tmp = cbmapiternext(villa->nodec, NULL)) != NULL){
+    pid = *(int *)tmp;
+    if(!vlnodecacheout(villa, pid)) err = TRUE;
+  }
+  if(!dpsetalign(villa->depot, 0)) err = TRUE;
+  if(!vldpputnum(villa->depot, VL_ROOTKEY, villa->root)) err = TRUE;
+  if(!vldpputnum(villa->depot, VL_LASTKEY, villa->last)) err = TRUE;
+  if(!vldpputnum(villa->depot, VL_LNUMKEY, villa->lnum)) err = TRUE;
+  if(!vldpputnum(villa->depot, VL_NNUMKEY, villa->nnum)) err = TRUE;
+  if(!vldpputnum(villa->depot, VL_RNUMKEY, villa->rnum)) err = TRUE;
+  if(!dpsetalign(villa->depot, VL_PAGEALIGN)) err = TRUE;
+  if(!dpmemsync(villa->depot)) err = TRUE;
+  return err ? FALSE : TRUE;
+}
+
+
+/* Synchronize updating contents on memory, not physically. */
+int vlmemflush(VILLA *villa){
+  int err, pid;
+  const char *tmp;
+  assert(villa);
+  if(!villa->wmode){
+    dpecodeset(DP_EMODE, __FILE__, __LINE__);
+    return FALSE;
+  }
+  if(villa->tran){
+    dpecodeset(DP_EMISC, __FILE__, __LINE__);
+    return FALSE;
+  }
+  err = FALSE;
+  cbmapiterinit(villa->leafc);
+  while((tmp = cbmapiternext(villa->leafc, NULL)) != NULL){
+    pid = *(int *)tmp;
+    if(!vlleafcacheout(villa, pid)) err = TRUE;
+  }
+  cbmapiterinit(villa->nodec);
+  while((tmp = cbmapiternext(villa->nodec, NULL)) != NULL){
+    pid = *(int *)tmp;
+    if(!vlnodecacheout(villa, pid)) err = TRUE;
+  }
+  if(!dpsetalign(villa->depot, 0)) err = TRUE;
+  if(!vldpputnum(villa->depot, VL_ROOTKEY, villa->root)) err = TRUE;
+  if(!vldpputnum(villa->depot, VL_LASTKEY, villa->last)) err = TRUE;
+  if(!vldpputnum(villa->depot, VL_LNUMKEY, villa->lnum)) err = TRUE;
+  if(!vldpputnum(villa->depot, VL_NNUMKEY, villa->nnum)) err = TRUE;
+  if(!vldpputnum(villa->depot, VL_RNUMKEY, villa->rnum)) err = TRUE;
+  if(!dpsetalign(villa->depot, VL_PAGEALIGN)) err = TRUE;
+  if(!dpmemflush(villa->depot)) err = TRUE;
+  return err ? FALSE : TRUE;
+}
+
+
 /* Refer to a volatile cache of a value of a record. */
 const char *vlgetcache(VILLA *villa, const char *kbuf, int ksiz, int *sp){
   VLLEAF *leaf;
@@ -1425,6 +1474,228 @@ const char *vlcurvalcache(VILLA *villa, int *sp){
 }
 
 
+/* Get a multiple cursor handle. */
+VLMULCUR *vlmulcuropen(VILLA *villa){
+  VLMULCUR *mulcur;
+  assert(villa);
+  if(villa->wmode){
+    dpecodeset(DP_EMODE, __FILE__, __LINE__);
+    return NULL;
+  }
+  CB_MALLOC(mulcur, sizeof(VLMULCUR));
+  mulcur->villa = villa;
+  mulcur->curleaf = -1;
+  mulcur->curknum = -1;
+  mulcur->curvnum = -1;
+  return mulcur;
+}
+
+
+/* Close a multiple cursor handle. */
+void vlmulcurclose(VLMULCUR *mulcur){
+  assert(mulcur);
+  free(mulcur);
+}
+
+
+/* Move a multiple cursor to the first record. */
+int vlmulcurfirst(VLMULCUR *mulcur){
+  VLMULCUR swap;
+  int rv;
+  assert(mulcur);
+  swap.curleaf = mulcur->villa->curleaf;
+  swap.curknum = mulcur->villa->curknum;
+  swap.curvnum = mulcur->villa->curvnum;
+  mulcur->villa->curleaf = mulcur->curleaf;
+  mulcur->villa->curknum = mulcur->curknum;
+  mulcur->villa->curvnum = mulcur->curvnum;
+  rv = vlcurfirst(mulcur->villa);
+  mulcur->curleaf = mulcur->villa->curleaf;
+  mulcur->curknum = mulcur->villa->curknum;
+  mulcur->curvnum = mulcur->villa->curvnum;
+  mulcur->villa->curleaf = swap.curleaf;
+  mulcur->villa->curknum = swap.curknum;
+  mulcur->villa->curvnum = swap.curvnum;
+  return rv;
+}
+
+
+/* Move a multiple cursor to the last record. */
+int vlmulcurlast(VLMULCUR *mulcur){
+  VLMULCUR swap;
+  int rv;
+  assert(mulcur);
+  swap.curleaf = mulcur->villa->curleaf;
+  swap.curknum = mulcur->villa->curknum;
+  swap.curvnum = mulcur->villa->curvnum;
+  mulcur->villa->curleaf = mulcur->curleaf;
+  mulcur->villa->curknum = mulcur->curknum;
+  mulcur->villa->curvnum = mulcur->curvnum;
+  rv = vlcurlast(mulcur->villa);
+  mulcur->curleaf = mulcur->villa->curleaf;
+  mulcur->curknum = mulcur->villa->curknum;
+  mulcur->curvnum = mulcur->villa->curvnum;
+  mulcur->villa->curleaf = swap.curleaf;
+  mulcur->villa->curknum = swap.curknum;
+  mulcur->villa->curvnum = swap.curvnum;
+  return rv;
+}
+
+
+/* Move a multiple cursor to the previous record. */
+int vlmulcurprev(VLMULCUR *mulcur){
+  VLMULCUR swap;
+  int rv;
+  assert(mulcur);
+  swap.curleaf = mulcur->villa->curleaf;
+  swap.curknum = mulcur->villa->curknum;
+  swap.curvnum = mulcur->villa->curvnum;
+  mulcur->villa->curleaf = mulcur->curleaf;
+  mulcur->villa->curknum = mulcur->curknum;
+  mulcur->villa->curvnum = mulcur->curvnum;
+  rv = vlcurprev(mulcur->villa);
+  mulcur->curleaf = mulcur->villa->curleaf;
+  mulcur->curknum = mulcur->villa->curknum;
+  mulcur->curvnum = mulcur->villa->curvnum;
+  mulcur->villa->curleaf = swap.curleaf;
+  mulcur->villa->curknum = swap.curknum;
+  mulcur->villa->curvnum = swap.curvnum;
+  return rv;
+}
+
+
+/* Move a multiple cursor to the next record. */
+int vlmulcurnext(VLMULCUR *mulcur){
+  VLMULCUR swap;
+  int rv;
+  assert(mulcur);
+  swap.curleaf = mulcur->villa->curleaf;
+  swap.curknum = mulcur->villa->curknum;
+  swap.curvnum = mulcur->villa->curvnum;
+  mulcur->villa->curleaf = mulcur->curleaf;
+  mulcur->villa->curknum = mulcur->curknum;
+  mulcur->villa->curvnum = mulcur->curvnum;
+  rv = vlcurnext(mulcur->villa);
+  mulcur->curleaf = mulcur->villa->curleaf;
+  mulcur->curknum = mulcur->villa->curknum;
+  mulcur->curvnum = mulcur->villa->curvnum;
+  mulcur->villa->curleaf = swap.curleaf;
+  mulcur->villa->curknum = swap.curknum;
+  mulcur->villa->curvnum = swap.curvnum;
+  return rv;
+}
+
+
+/* Move a multiple cursor to a position around a record. */
+int vlmulcurjump(VLMULCUR *mulcur, const char *kbuf, int ksiz, int jmode){
+  VLMULCUR swap;
+  int rv;
+  assert(mulcur);
+  swap.curleaf = mulcur->villa->curleaf;
+  swap.curknum = mulcur->villa->curknum;
+  swap.curvnum = mulcur->villa->curvnum;
+  mulcur->villa->curleaf = mulcur->curleaf;
+  mulcur->villa->curknum = mulcur->curknum;
+  mulcur->villa->curvnum = mulcur->curvnum;
+  rv = vlcurjump(mulcur->villa, kbuf, ksiz, jmode);
+  mulcur->curleaf = mulcur->villa->curleaf;
+  mulcur->curknum = mulcur->villa->curknum;
+  mulcur->curvnum = mulcur->villa->curvnum;
+  mulcur->villa->curleaf = swap.curleaf;
+  mulcur->villa->curknum = swap.curknum;
+  mulcur->villa->curvnum = swap.curvnum;
+  return rv;
+}
+
+
+/* Get the key of the record where a multiple cursor is. */
+char *vlmulcurkey(VLMULCUR *mulcur, int *sp){
+  VLMULCUR swap;
+  char *rv;
+  assert(mulcur);
+  swap.curleaf = mulcur->villa->curleaf;
+  swap.curknum = mulcur->villa->curknum;
+  swap.curvnum = mulcur->villa->curvnum;
+  mulcur->villa->curleaf = mulcur->curleaf;
+  mulcur->villa->curknum = mulcur->curknum;
+  mulcur->villa->curvnum = mulcur->curvnum;
+  rv = vlcurkey(mulcur->villa, sp);
+  mulcur->curleaf = mulcur->villa->curleaf;
+  mulcur->curknum = mulcur->villa->curknum;
+  mulcur->curvnum = mulcur->villa->curvnum;
+  mulcur->villa->curleaf = swap.curleaf;
+  mulcur->villa->curknum = swap.curknum;
+  mulcur->villa->curvnum = swap.curvnum;
+  return rv;
+}
+
+
+/* Get the value of the record where a multiple cursor is. */
+char *vlmulcurval(VLMULCUR *mulcur, int *sp){
+  VLMULCUR swap;
+  char *rv;
+  assert(mulcur);
+  swap.curleaf = mulcur->villa->curleaf;
+  swap.curknum = mulcur->villa->curknum;
+  swap.curvnum = mulcur->villa->curvnum;
+  mulcur->villa->curleaf = mulcur->curleaf;
+  mulcur->villa->curknum = mulcur->curknum;
+  mulcur->villa->curvnum = mulcur->curvnum;
+  rv = vlcurval(mulcur->villa, sp);
+  mulcur->curleaf = mulcur->villa->curleaf;
+  mulcur->curknum = mulcur->villa->curknum;
+  mulcur->curvnum = mulcur->villa->curvnum;
+  mulcur->villa->curleaf = swap.curleaf;
+  mulcur->villa->curknum = swap.curknum;
+  mulcur->villa->curvnum = swap.curvnum;
+  return rv;
+}
+
+
+/* Refer to volatile cache of the key of the record where a multiple cursor is. */
+const char *vlmulcurkeycache(VLMULCUR *mulcur, int *sp){
+  VLMULCUR swap;
+  const char *rv;
+  assert(mulcur);
+  swap.curleaf = mulcur->villa->curleaf;
+  swap.curknum = mulcur->villa->curknum;
+  swap.curvnum = mulcur->villa->curvnum;
+  mulcur->villa->curleaf = mulcur->curleaf;
+  mulcur->villa->curknum = mulcur->curknum;
+  mulcur->villa->curvnum = mulcur->curvnum;
+  rv = vlcurkeycache(mulcur->villa, sp);
+  mulcur->curleaf = mulcur->villa->curleaf;
+  mulcur->curknum = mulcur->villa->curknum;
+  mulcur->curvnum = mulcur->villa->curvnum;
+  mulcur->villa->curleaf = swap.curleaf;
+  mulcur->villa->curknum = swap.curknum;
+  mulcur->villa->curvnum = swap.curvnum;
+  return rv;
+}
+
+
+/* Refer to volatile cache of the value of the record where a multiple cursor is. */
+const char *vlmulcurvalcache(VLMULCUR *mulcur, int *sp){
+  VLMULCUR swap;
+  const char *rv;
+  assert(mulcur);
+  swap.curleaf = mulcur->villa->curleaf;
+  swap.curknum = mulcur->villa->curknum;
+  swap.curvnum = mulcur->villa->curvnum;
+  mulcur->villa->curleaf = mulcur->curleaf;
+  mulcur->villa->curknum = mulcur->curknum;
+  mulcur->villa->curvnum = mulcur->curvnum;
+  rv = vlcurvalcache(mulcur->villa, sp);
+  mulcur->curleaf = mulcur->villa->curleaf;
+  mulcur->curknum = mulcur->villa->curknum;
+  mulcur->curvnum = mulcur->villa->curvnum;
+  mulcur->villa->curleaf = swap.curleaf;
+  mulcur->villa->curknum = swap.curknum;
+  mulcur->villa->curvnum = swap.curvnum;
+  return rv;
+}
+
+
 
 /*************************************************************************************************
  * private objects
@@ -1611,9 +1882,7 @@ static int vlleafcacheout(VILLA *villa, int id){
   assert(villa && id >= VL_LEAFIDMIN);
   if(!(leaf = (VLLEAF *)cbmapget(villa->leafc, (char *)&id, sizeof(int), NULL))) return FALSE;
   err = FALSE;
-  if(leaf->dirty){
-    if(!vlleafsave(villa, leaf)) err = TRUE;
-  }
+  if(leaf->dirty && !vlleafsave(villa, leaf)) err = TRUE;
   ln = CB_LISTNUM(leaf->recs);
   for(i = 0; i < ln; i++){
     recp = (VLREC *)CB_LISTVAL(leaf->recs, i, NULL);
@@ -1979,9 +2248,7 @@ static int vlnodecacheout(VILLA *villa, int id){
   assert(villa && id >= VL_NODEIDMIN);
   if(!(node = (VLNODE *)cbmapget(villa->nodec, (char *)&id, sizeof(int), NULL))) return FALSE;
   err = FALSE;
-  if(node->dirty){
-    if(!vlnodesave(villa, node)) err = TRUE;
-  }
+  if(node->dirty && !vlnodesave(villa, node)) err = TRUE;
   ln = CB_LISTNUM(node->idxs);
   for(i = 0; i < ln; i++){
     idxp = (VLIDX *)CB_LISTVAL(node->idxs, i, NULL);
@@ -2255,17 +2522,6 @@ static VLREC *vlrecsearch(VILLA *villa, VLLEAF *leaf, const char *kbuf, int ksiz
 }
 
 
-/* Set the size of the free block pool of a database handle. */
-int vlsetfbpsiz(VILLA *villa, int size){
-  assert(villa && size >= 0);
-  if(!villa->wmode){
-    dpecodeset(DP_EMODE, __FILE__, __LINE__);
-    return FALSE;
-  }
-  return dpsetfbpsiz(villa->depot, size);
-}
-
-
 /* Get flags of a database. */
 int vlgetflags(VILLA *villa){
   assert(villa);
