@@ -61,11 +61,12 @@ int docreate(const char *name, int cmode);
 int doput(const char *name, const char *kbuf, int ksiz, const char *vbuf, int vsiz,
           int dmode, VLCFUNC cmp);
 int doout(const char *name, const char *kbuf, int ksiz, VLCFUNC cmp, int lb);
-int doget(const char *name, const char *kbuf, int ksiz, VLCFUNC cmp, int lb, int ox, int nb);
-int dolist(const char *name, const char *tbuf, int tsiz, const char *bbuf, int bsiz,
+int doget(const char *name, int opts, const char *kbuf, int ksiz, VLCFUNC cmp,
+          int lb, int ox, int nb);
+int dolist(const char *name, int opts, const char *tbuf, int tsiz, const char *bbuf, int bsiz,
            VLCFUNC cmp, int ki, int kb, int vb, int ox, int gt, int lt, int max, int desc);
 int dooptimize(const char *name);
-int doinform(const char *name);
+int doinform(const char *name, int opts);
 int doremove(const char *name);
 int dorepair(const char *name, VLCFUNC cmp);
 int doexportdb(const char *name, const char *file, VLCFUNC cmp);
@@ -123,11 +124,11 @@ void usage(void){
   fprintf(stderr, "  %s create [-cz|-cy|-cx] name\n", progname);
   fprintf(stderr, "  %s put [-kx|-ki] [-vx|-vi|-vf] [-keep|-cat|-dup] name key val\n", progname);
   fprintf(stderr, "  %s out [-l] [-kx|-ki] name key\n", progname);
-  fprintf(stderr, "  %s get [-l] [-kx|-ki] [-ox] [-n] name key\n", progname);
-  fprintf(stderr, "  %s list [-k|-v] [-kx|-ki] [-ox] [-top key] [-bot key] [-gt] [-lt]"
+  fprintf(stderr, "  %s get [-nl] [-l] [-kx|-ki] [-ox] [-n] name key\n", progname);
+  fprintf(stderr, "  %s list [-nl] [-k|-v] [-kx|-ki] [-ox] [-top key] [-bot key] [-gt] [-lt]"
           " [-max num] [-desc] name\n", progname);
   fprintf(stderr, "  %s optimize name\n", progname);
-  fprintf(stderr, "  %s inform name\n", progname);
+  fprintf(stderr, "  %s inform [-nl] name\n", progname);
   fprintf(stderr, "  %s remove name\n", progname);
   fprintf(stderr, "  %s repair [-ki] name\n", progname);
   fprintf(stderr, "  %s exportdb [-ki] name file\n", progname);
@@ -332,8 +333,9 @@ int runout(int argc, char **argv){
 /* parse arguments of get command */
 int runget(int argc, char **argv){
   char *name, *key, *kbuf;
-  int i, lb, kx, ki, ox, nb, ksiz, rv;
+  int i, opts, lb, kx, ki, ox, nb, ksiz, rv;
   name = NULL;
+  opts = 0;
   lb = FALSE;
   kx = FALSE;
   ki = FALSE;
@@ -342,7 +344,9 @@ int runget(int argc, char **argv){
   key = NULL;
   for(i = 2; i < argc; i++){
     if(!name && argv[i][0] == '-'){
-      if(!strcmp(argv[i], "-l")){
+      if(!strcmp(argv[i], "-nl")){
+        opts |= VL_ONOLCK;
+      } else if(!strcmp(argv[i], "-l")){
         lb = TRUE;
       } else if(!strcmp(argv[i], "-kx")){
         kx = TRUE;
@@ -373,7 +377,7 @@ int runget(int argc, char **argv){
     ksiz = -1;
   }
   if(kbuf){
-    rv = doget(name, kbuf, ksiz, ki ? VL_CMPINT : VL_CMPLEX, lb, ox, nb);
+    rv = doget(name, opts, kbuf, ksiz, ki ? VL_CMPINT : VL_CMPLEX, lb, ox, nb);
   } else {
     fprintf(stderr, "%s: out of memory\n", progname);
     rv = 1;
@@ -386,8 +390,9 @@ int runget(int argc, char **argv){
 /* parse arguments of list command */
 int runlist(int argc, char **argv){
   char *name, *top, *bot, *tbuf, *bbuf, *nstr;
-  int i, kb, vb, kx, ki, ox, gt, lt, max, desc, tsiz, bsiz, rv;
+  int i, opts, kb, vb, kx, ki, ox, gt, lt, max, desc, tsiz, bsiz, rv;
   name = NULL;
+  opts = 0;
   kb = FALSE;
   vb = FALSE;
   kx = FALSE;
@@ -402,7 +407,9 @@ int runlist(int argc, char **argv){
   nstr = NULL;
   for(i = 2; i < argc; i++){
     if(!name && argv[i][0] == '-'){
-      if(!strcmp(argv[i], "-k")){
+      if(!strcmp(argv[i], "-nl")){
+        opts |= VL_ONOLCK;
+      } else if(!strcmp(argv[i], "-k")){
         kb = TRUE;
       } else if(!strcmp(argv[i], "-v")){
         vb = TRUE;
@@ -455,7 +462,7 @@ int runlist(int argc, char **argv){
       bsiz = strlen(bbuf);
     }
   }
-  rv = dolist(name, tbuf, tsiz, bbuf, bsiz, ki ? VL_CMPINT : VL_CMPLEX, ki,
+  rv = dolist(name, opts, tbuf, tsiz, bbuf, bsiz, ki ? VL_CMPINT : VL_CMPLEX, ki,
               kb, vb, ox, gt, lt, max, desc);
   free(tbuf);
   free(bbuf);
@@ -486,11 +493,16 @@ int runoptimize(int argc, char **argv){
 /* parse arguments of inform command */
 int runinform(int argc, char **argv){
   char *name;
-  int i, rv;
+  int i, opts, rv;
   name = NULL;
+  opts = 0;
   for(i = 2; i < argc; i++){
     if(!name && argv[i][0] == '-'){
-      usage();
+      if(!strcmp(argv[i], "-nl")){
+        opts |= VL_ONOLCK;
+      } else {
+        usage();
+      }
     } else if(!name){
       name = argv[i];
     } else {
@@ -498,7 +510,7 @@ int runinform(int argc, char **argv){
     }
   }
   if(!name) usage();
-  rv = doinform(name);
+  rv = doinform(name, opts);
   return rv;
 }
 
@@ -695,12 +707,13 @@ int doout(const char *name, const char *kbuf, int ksiz, VLCFUNC cmp, int lb){
 
 
 /* perform get command */
-int doget(const char *name, const char *kbuf, int ksiz, VLCFUNC cmp, int lb, int ox, int nb){
+int doget(const char *name, int opts, const char *kbuf, int ksiz, VLCFUNC cmp,
+          int lb, int ox, int nb){
   VILLA *villa;
   CBLIST *vals;
   char *vbuf;
   int vsiz;
-  if(!(villa = vlopen(name, VL_OREADER, cmp))){
+  if(!(villa = vlopen(name, VL_OREADER | opts, cmp))){
     pdperror(name);
     return 1;
   }
@@ -743,12 +756,12 @@ int doget(const char *name, const char *kbuf, int ksiz, VLCFUNC cmp, int lb, int
 
 
 /* perform list command */
-int dolist(const char *name, const char *tbuf, int tsiz, const char *bbuf, int bsiz,
+int dolist(const char *name, int opts, const char *tbuf, int tsiz, const char *bbuf, int bsiz,
            VLCFUNC cmp, int ki, int kb, int vb, int ox, int gt, int lt, int max, int desc){
   VILLA *villa;
   char *kbuf, *vbuf;
   int ksiz, vsiz, show, rv;
-  if(!(villa = vlopen(name, VL_OREADER, cmp))){
+  if(!(villa = vlopen(name, VL_OREADER | opts, cmp))){
     pdperror(name);
     return 1;
   }
@@ -867,10 +880,10 @@ int dooptimize(const char *name){
 
 
 /* perform inform command */
-int doinform(const char *name){
+int doinform(const char *name, int opts){
   VILLA *villa;
   char *tmp;
-  if(!(villa = vlopen(name, VL_OREADER, VL_CMPLEX))){
+  if(!(villa = vlopen(name, VL_OREADER | opts, VL_CMPLEX))){
     pdperror(name);
     return 1;
   }
